
#include <windows.h>  
#include <commctrl.h>
#include "crdrag.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "App";
LPCTSTR lpszTitle   = "TreeView_CreateDragImage()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

VOID AddItemsToTree( HWND hTree );

#define IMAGESIZE 16

LPCTSTR lpszData[4] = { "Circle", "Rectangle", "Cross", "Check" };

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static TV_HITTESTINFO tvHitTest;
static POINT          ptHotSpot;
static HTREEITEM      hDragItem = NULL;
static HWND           hTree     = NULL;
static HIMAGELIST     hCursors  = NULL;
static HIMAGELIST     hDragList = NULL;

   switch( uMsg )
   {
      case WM_CREATE  :
              {
                 HIMAGELIST hImage;
                 ICONINFO   info;

                 InitCommonControls();

                 hCursors = ImageList_Create( 32, 32, ILC_COLOR4 | ILC_MASK, 1, 1 ); 
                 hImage   = ImageList_Create( IMAGESIZE, IMAGESIZE, 
                                              ILC_COLOR4 | ILC_MASK, 4, 1 ); 
                 hTree = CreateWindowEx( WS_EX_CLIENTEDGE, WC_TREEVIEW, "",
                                         WS_CHILD | WS_BORDER | WS_VISIBLE | 
                                         WS_VSCROLL | TVS_HASLINES | 
                                         TVS_HASBUTTONS | TVS_LINESATROOT,
                                         10, 100, 100, 100,
                                         hWnd,
                                         (HMENU)1,
                                         hInst,
                                         NULL);

                 if ( hTree )
                 {
                    int i;

                    // Associate the image list with the tree view.
                    //.............................................
                    TreeView_SetImageList( hTree, hImage, TVSIL_NORMAL );

                    // Add the images to the tree view image list.
                    //............................................
                    for ( i=0; i<4; i++ )
                       ImageList_AddIcon( hImage, LoadIcon( hInst, lpszData[i] ) );

                    AddItemsToTree( hTree );
                 }

                 // Create drag cursor icon.
                 //.........................
                 ImageList_AddIcon( hCursors, LoadCursor( NULL, IDC_ARROW ) );

                 // Retrieve the hot spot information from the cursor.
                 //...................................................
                 GetIconInfo( LoadCursor( NULL, IDC_ARROW ), &info );
                 ptHotSpot.x = info.xHotspot;
                 ptHotSpot.y = info.yHotspot;

                 // Clean up the bitmaps from GetIconInfo().
                 //.........................................
                 DeleteObject( info.hbmMask );
                 DeleteObject( info.hbmColor );
              }
              break;

      case WM_SIZE :
              if ( hTree )
                 MoveWindow( hTree, 0, 0, LOWORD( lParam ), HIWORD( lParam ), FALSE );
              break;


      case WM_NOTIFY :
              if ( wParam == 1 )
              {
                 NM_TREEVIEW* pNM = (NM_TREEVIEW*)lParam;

                 switch ( pNM->hdr.code )
                 {
                    case TVN_BEGINDRAG :                 
                           {
                              RECT rc;

                              // Create the drag image.
                              //.......................
                              hDragList = TreeView_CreateDragImage( hTree, pNM->itemNew.hItem );

                              TreeView_GetItemRect( hTree, pNM->itemNew.hItem, &rc, FALSE );

                              // Start dragging and set the drag cursor.
                              //........................................
                              ImageList_BeginDrag( hDragList, 0, 
                                            pNM->ptDrag.x-rc.left-IMAGESIZE, 
                                            pNM->ptDrag.y-rc.top );
                              ImageList_SetDragCursorImage( hCursors, 0, 
                                            pNM->ptDrag.x-rc.left-ptHotSpot.x-IMAGESIZE, 
                                            pNM->ptDrag.y-rc.top-ptHotSpot.y );
                              ImageList_DragEnter( hTree, pNM->ptDrag.x, pNM->ptDrag.y ); 

                              // Capture the mouse and hide the normal cursor.
                              //..............................................
                              SetCapture( hWnd );
                              ShowCursor( FALSE );

                              hDragItem = pNM->itemNew.hItem;
                           }
                           break;

                 } 
              }
              break;

      case WM_MOUSEMOVE :
              // Move the drag cursor if dragging.
              //..................................
              if ( hDragList )
              {
                 tvHitTest.pt.x = LOWORD( lParam );
                 tvHitTest.pt.y = HIWORD( lParam );

                 // Check to see if the item is over a tree view item,
                 // if it is, then it is hilighted.
                 //...................................................
                 TreeView_HitTest( hTree, &tvHitTest );
                 if ( tvHitTest.hItem != TreeView_GetDropHilight( hTree ) )
                 {
                    // Clear the drag image.
                    //......................
                    ImageList_DragLeave( hTree );

                    // Unhilight the old item, hilight the new one.
                    //.............................................
                    TreeView_SelectDropTarget( hTree, NULL );
                    TreeView_SelectDropTarget( hTree, tvHitTest.hItem );

                    // Show the drag image.
                    //.....................
                    ImageList_DragEnter( hTree, LOWORD( lParam ), HIWORD( lParam ) ); 
                 }
                 else   
                    ImageList_DragMove( LOWORD( lParam ), HIWORD( lParam ) );
              }
              break;


      case WM_LBUTTONUP :
              // If dragging, drop the object in its new place.
              //...............................................
              if ( hDragList )
              {
                 HTREEITEM hDropItem;

                 // Stop the dragging process.
                 //...........................
                 ImageList_DragLeave( hTree );
                 ImageList_EndDrag();
                 ImageList_Destroy( hDragList );
                 hDragList = NULL;
                 ReleaseCapture();
                 ShowCursor( TRUE );

                 // Find out the item that was last hilighted.
                 //...........................................
                 hDropItem = TreeView_GetDropHilight( hTree );

                 // Unselect the drop hilight item.
                 //................................
                 TreeView_SelectDropTarget( hTree, NULL );

                 // Item was dropped, move the item in the tree.
                 //.............................................
                 if ( hDropItem )
                 {
                    TV_INSERTSTRUCT tv;
                    TV_ITEM         tvDropItem;
                    char            szName[40];
                    
                    // Retrieve the Drag Item.
                    //........................  
                    tv.item.hItem      = hDragItem;
                    tv.item.mask       = TVIF_TEXT | TVIF_IMAGE | 
                                         TVIF_SELECTEDIMAGE | TVIF_PARAM;
                    tv.item.pszText    = szName;
                    tv.item.cchTextMax = sizeof( szName );
                    TreeView_GetItem( hTree, &tv.item );

                    // Retrieve the Drop Item.
                    //........................
                    tvDropItem.hItem      = hDropItem;
                    tvDropItem.mask       = TVIF_PARAM;
                    TreeView_GetItem( hTree, &tvDropItem );

                    // Only let level 1 items go to level 1 and
                    // level 2 items go to level 2.
                    //.........................................
                    if ( tv.item.lParam && tvDropItem.lParam )
                    {
                       tv.hParent = TreeView_GetParent( hTree, hDropItem );
                       tv.hInsertAfter = hDropItem;
                    }
                    else if ( tv.item.lParam && !tvDropItem.lParam )
                    {
                       tv.hParent = hDropItem;
                       tv.hInsertAfter = TVI_LAST;
                    }
                    else if ( !tv.item.lParam && tvDropItem.lParam )
                    {
                       tv.hParent = TVI_ROOT;
                       tv.hInsertAfter = TreeView_GetParent( hTree, hDropItem );
                    }
                    else if ( !tv.item.lParam && !tvDropItem.lParam )
                    {
                       tv.hParent = TVI_ROOT;
                       tv.hInsertAfter = hDropItem;
                    }

                    // Insert the dragged item into the tree
                    // in the new location.
                    //......................................
                    tv.hParent = TreeView_InsertItem( hTree, &tv );

                    // Move the children if there are any.
                    //....................................
                    if ( !tv.item.lParam )
                    {
                       tv.hInsertAfter = TVI_LAST;
                       tv.item.hItem = TreeView_GetChild( hTree, hDragItem );
                       while ( tv.item.hItem )
                       {
                          TreeView_GetItem( hTree, &tv.item );
                          TreeView_InsertItem( hTree, &tv );
                          tv.item.hItem = TreeView_GetNextSibling( hTree, tv.item.hItem );
                       }
                    }

                    // Delete the original item from the tree,
                    // this is because we are moving the item.
                    //........................................
                    TreeView_DeleteItem( hTree, hDragItem ); 
                 }

                 // Update the tree view.
                 //......................
                 InvalidateRect( hTree, NULL, TRUE );

              }
              break;


      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_RESET :
                        // Clear the tree view.
                        //.....................
                        TreeView_DeleteAllItems( hTree );

                        // Add the items to the tree.
                        //...........................
                        AddItemsToTree( hTree );
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              if ( TreeView_GetImageList( hTree, TVSIL_NORMAL ) )
                 ImageList_Destroy( TreeView_GetImageList( hTree, TVSIL_NORMAL ) );

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


VOID AddItemsToTree( HWND hTree )
{
   int             i, j;
   char            szTmp[40];
   TV_INSERTSTRUCT tv;
   HTREEITEM       hParent;

   tv.hInsertAfter = TVI_LAST;
   tv.item.mask    = TVIF_TEXT | TVIF_IMAGE | 
                     TVIF_SELECTEDIMAGE | TVIF_PARAM;

   // Add the items to the tree view.
   //................................
   for ( i=0; i<20; i++ )
   {
      wsprintf( szTmp, "%s %d", lpszData[0], i+1 );

      tv.hParent      = TVI_ROOT;
      tv.item.pszText = szTmp;
      tv.item.iImage  = 0;
      tv.item.iSelectedImage = 0;
      tv.item.lParam         = 0;

      hParent = TreeView_InsertItem( hTree, &tv );
      tv.hParent = hParent;

      // Add child items.
      //.................
      for( j=1; j<4; j++ )
      {
         tv.item.pszText = (LPTSTR)lpszData[j];
         tv.item.iImage  = j;
         tv.item.iSelectedImage = j;
         tv.item.lParam         = 1;

         TreeView_InsertItem( hTree, &tv );
      }
   }
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}


